/*
 * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//resources/src/java/org/apache/commons/resources/impl/BasicMessageList.java,v 1.14 2004/09/07 23:34:46 jmitchell Exp $
 * $Revision: 1.14 $
 * $Date: 2004-09-07 19:34:46 -0400 (Tue, 07 Sep 2004) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.resources.Message;
import org.apache.commons.resources.MessageList;

/**
 * A basic implementation of a MessageList.
 * <p>
 * Orginally based on org.apache.http.action.ActionMessages, Revision 1.7.
 */
public class BasicMessageList implements Serializable, MessageList {

    /**
     * Compares MessageItems.
     */
    private static final Comparator actionItemComparator = new Comparator() {
        public int compare(Object o1, Object o2) {
            return ((MessageItem) o1).getOrder() - ((MessageItem) o2).getOrder();
        }
    };
    
    /**
     * Have the messages been retrieved from this object?
     */
    private boolean accessed = false;

    /**
     * The accumulated set of <code>Message</code> objects (represented
     * as a List) for each property, keyed by property name.
     */
    protected Map messages = new HashMap();

    /**
     * The current number of the property/key being added.  This is used
     * to maintain the order messages are added.
     */
    private int count = 0;

    // --------------------------------------------------------- Public Methods

    /**
     * Create an empty <code>MessageList</code> object.
     */
    public BasicMessageList() {
        super();
    }

    /**
     * Create an <code>MessageList</code> object initialized to use
     * the given value for the "global" message key.
     *
     * @param globalMessageKey The new default global message key
     */
    public BasicMessageList(String globalMessageKey) {
        super();
        this.setGlobalMessageKey(globalMessageKey);
    }

    /**
     * Create an <code>MessageList</code> object initialized with the given
     * messages.
     *
     * @param messages The messages to be initially added to this object.
     */
    public BasicMessageList(MessageList messages) {
        super();
        this.add(messages);
    }

    /**
     * Create an <code>MessageList</code> object initialized with the given
     * messages and the given global message key.
     *
     * @param globalMessageKey The new default global message key
     * @param messages The messages to be initially added to this object.
     * 
     * @TODO - FIXME - globalMessageKey is ignored
     */
    public BasicMessageList(String globalMessageKey, MessageList messages) {
        super();
        this.add(messages);
    }

    /**
     * The "global" message key for this MessageList
     * [GLOBAL_MESSAGE_KEY].
     */
    private String globalMessageKey = GLOBAL_MESSAGE_KEY;

    // See interface for JavaDoc
    public String getGlobalMessageKey() {
        return this.globalMessageKey;
    }

    // See interface for JavaDoc
    public void setGlobalMessageKey(String globalMessageKey) {
        this.globalMessageKey = globalMessageKey;
    }

    // See interface for JavaDoc
    public void add(String property, Message message) {

        MessageItem item = (MessageItem) messages.get(property);
        List list = null;

        if (item == null) {
            list = new ArrayList();
            item = new MessageItem(list, this.count++);

            messages.put(property, item);
        } else {
            list = item.getList();
        }

        list.add(message);

    }


    // See interface for JavaDoc
    public void add(Message message) {
        this.add(MessageList.GLOBAL_MESSAGE_KEY, message);
    }


    // See interface for JavaDoc
    public void add(MessageList messageList) {
        // loop over properties
        Iterator props = messageList.properties();
        while (props.hasNext()) {
            String property = (String) props.next();

            // loop over messages for each property
            Iterator msgs = messageList.get(property);
            while (msgs.hasNext()) {
                Message msg = (Message) msgs.next();
                this.add(property, msg);
            }

        }
    }

    // See interface for JavaDoc
    public void clear() {
        this.messages.clear();
    }
    
    // See interface for JavaDoc
    public boolean isAccessed() {
        return this.accessed;
    }

    // See interface for JavaDoc
    public boolean isEmpty() {
        return messages.isEmpty();
    }

    // See interface for JavaDoc
    public Iterator get() {
        this.accessed = true;
        
        if (messages.isEmpty()) {
            return Collections.EMPTY_LIST.iterator();
        }

        ArrayList results = new ArrayList();
        ArrayList actionItems = new ArrayList();

        for (Iterator i = messages.values().iterator(); i.hasNext();) {
            actionItems.add(i.next());
        }

        // Sort MessageItems based on the initial order the
        // property/key was added to MessageList.
        Collections.sort(actionItems, actionItemComparator);

        for (Iterator i = actionItems.iterator(); i.hasNext();) {
            MessageItem ami = (MessageItem) i.next();

            for (Iterator msgs = ami.getList().iterator(); msgs.hasNext();) {
                results.add(msgs.next());
            }
        }

        return results.iterator();
    }

    // See interface for JavaDoc
    public Iterator get(String property) {
        this.accessed = true;
        
        MessageItem item = (MessageItem) messages.get(property);
        
        return (item == null)
            ? Collections.EMPTY_LIST.iterator()
            : item.getList().iterator();
    }

    // See interface for JavaDoc
    public Iterator properties() {
        return messages.keySet().iterator();
    }

    // See interface for JavaDoc
    public int size() {

        int total = 0;

        for (Iterator i = messages.values().iterator(); i.hasNext();) {
            MessageItem ami = (MessageItem) i.next();
            total += ami.getList().size();
        }

        return total;
    }

    // See interface for JavaDoc
    public int size(String property) {

        MessageItem ami = (MessageItem) messages.get(property);

        return (ami == null) ? 0 : ami.getList().size();
    }
    
    /**
     * Returns a String representation of this MessageList's 
     * [property name]=[message list] mapping.
     * @see java.lang.Object#toString()
     */    
    public String toString() {
        return this.messages.toString();
    }

    // See interface for JavaDoc
    protected static class MessageItem implements Serializable {

        /**
         * The list of <code>Message</code>s.
         */
        private List list = null;

        /**
         * The position in the list of messages.
         */
        private int order = 0;

        public MessageItem(List list, int order) {
            this.list = list;
            this.order = order;
        }

        public List getList() {
            return list;
        }

        public void setList(List list) {
            this.list = list;
        }

        public int getOrder() {
            return order;
        }

        public void setOrder(int order) {
            this.order = order;
        }

    }

}
